@@ -19,6 +19,33 @@
   return(geo_envir) ## some elements may still be NULL
 }
 
+.calc_IMRF_Qmat <- function(pars, grid_arglist, kappa) {
+  if (is.null(spde_info <- pars$model)) { 
+    crossfac_Q <- .IMRFcrossfactor(xstwm=length(grid_arglist[[1]]), ystwm=length(grid_arglist[[2]]),kappa=kappa)
+    sparse_Qmat <- crossprod(crossfac_Q) # same relation as for t_chol_Q ## automatically dsCMatrix
+  } else {
+    if (inherits(spde_info,"inla.spde2")) { # F I X M E only for d=2 
+      if (pars$SPDE_alpha==2) { # nu=1, alpha=nu+d/2=2
+        Cmat <- spde_info$param.inla$M0
+        Gmat <- spde_info$param.inla$M1
+        Kmat <- kappa^2 * Cmat +Gmat
+        if (isDiagonal(Cmat)) {
+          # solvesqC <- Cmat
+          # solvesqC@x <- 1/sqrt(solvesqC@x)
+          # crossfac_Q <- Kmat %*% solvesqC
+          tcrossfac_Q <- .Matrix_times_Dvec(Kmat, 1/sqrt(Cmat@x))
+          sparse_Qmat <- tcrossprod(tcrossfac_Q) # same relation as for t_chol_Q ## automatically dsCMatrix
+        } else {stop("Cmat does not appear to be diagonal")}
+      } else if (pars$SPDE_alpha==1) { # nu=0, alpha=nu+d/2=1
+        Cmat <- spde_info$param.inla$M0
+        Gmat <- spde_info$param.inla$M2
+        sparse_Qmat <- as(kappa^2 * Cmat +Gmat,"symmetricMatrix")
+      } 
+    } else stop("Unhandled model class for IMRF")
+  }
+  return(sparse_Qmat)
+}
+
 .assign_geoinfo_and_LMatrices_but_ranCoefs <- function(processed, corr_types, spatial_terms, 
                                                        ranPars, control.dist, argsfordesignL) {
   # * assigns geo_envir <- .get_geo_info(...)
@@ -60,11 +87,12 @@
           # fixme remove $d from symSVD to avoid later confusions ?
         }
         # symSVD may be modified below
-      } else if (corr_type =="MRF") {
+      } else if (corr_type =="IMRF") {
         kappa <- .get_cP_stuff(ranPars,"kappa",which=char_rd)
         pars <- attr(attr(spatial_term,"type"),"pars")
-        crossfac_Q <- .MRFcrossfactor(xsteps=pars$nd, ysteps=pars$nd, margin=pars$m,kappa=kappa)
-        sparse_Qmat <- crossprod(crossfac_Q) # same relation as for t_chol_Q ## automatically dsCMatrix
+        sparse_Qmat <- .calc_IMRF_Qmat(pars, 
+                                       grid_arglist=attr(attr(processed$ZAlist,"AMatrices")[[char_rd]],"grid_arglist"), # promise for spde case
+                                       kappa)
         if ( ! processed$sparsePrecisionBOOL) {
           cov_info_mat <- chol2inv(chol(sparse_Qmat)) # solve(sparse_Qmat) 
         }
@@ -136,17 +164,23 @@
         } else if (corr_type=="AR1") { 
           types <- processed$AUGI0_ZX$envir$finertypes
           ARphi <- .get_cP_stuff(ranPars,"ARphi",which=char_rd)
-          seq_n_u_h <- diff(processed$cum_n_u_h)
-          names(seq_n_u_h) <- types
           ## the dim of Q *HAS* to match that of ZA
           AR1_block_n_u_h_s <- attr(processed$ZAlist[[rd]],"AR1_block_n_u_h_s") # block size[S for single AR1 term if nested]
-          t_chol_Q <- sapply(AR1_block_n_u_h_s, .calc_AR1_t_chol_Q_block,ARphi=ARphi) ## L block(s) of given size(s)
-          ## bdiag required if nested AR1 term:
-          Lunique <- Matrix::bdiag(lapply(t_chol_Q, solve )) # sparse triangular solve 
+          ilist <- jlist <- xlist <- vector("list",length(AR1_block_n_u_h_s))
+          cum_ncol <- 0L
+          for (bloc in seq_along(AR1_block_n_u_h_s)) {
+            triplets <- .calc_AR1_t_chol_Q_block(AR1_block_n_u_h_s[[bloc]], ARphi=ARphi) ## L block(s) of given size(s)
+            ilist[[bloc]] <- cum_ncol + triplets$i
+            jlist[[bloc]] <- cum_ncol + triplets$j 
+            xlist[[bloc]] <- triplets$x
+            cum_ncol <- cum_ncol + AR1_block_n_u_h_s[[bloc]]
+          }
+          t_chol_Q <- sparseMatrix(dims = c(cum_ncol,cum_ncol), i=unlist(ilist), j=unlist(jlist),
+                                   x = unlist(xlist), triangular=TRUE)
+          processed$AUGI0_ZX$envir$precisionFactorList[[rd]] <- list(chol_Q=t(t_chol_Q), # Linv
+                                                                     Qmat=crossprod(t_chol_Q))
+          Lunique <- solve(t_chol_Q)
           attr(Lunique, "type") <- "from_AR1_specific_code"
-          sparse_Qmat <- Matrix::bdiag(lapply(t_chol_Q, crossprod ))
-          processed$AUGI0_ZX$envir$precisionFactorList[[rd]] <- list(chol_Q=Matrix::bdiag(lapply(t_chol_Q, t )), # Linv
-                                                                     Qmat=sparse_Qmat)
         } else if (corr_type %in% c("Matern","Cauchy")) {
           ## at this point cov_info_mat is a dist object !
           cov_info_mat <- as.matrix(cov_info_mat)
@@ -156,24 +190,43 @@
           # cov_info_mat <- cov_info_mat[ZAnames,ZAnames]
           # but this would require controlling the names of Z to be those of cov_info_mat (spMMFactorList_locfn() does not care for that)
           sparse_Qmat <- as_precision(cov_info_mat)$matrix
-        } else if (corr_type== "MRF") {
+        } else if (corr_type== "IMRF") {
           # Remember that we need dtCMatrix'es 'chol_Q' so that bdiag() gives a dtCMatrix
           # Hence use next general code to produce precisionFactorList[[rd]]
         } else {stop("Some error occurred (inner estimation of adjacency rho with requested sparse precision ?)")}     
         ## (2) Builds from sparse_Qmat if not already available
-        if (corr_type != "AR1") { ## General code for "Matern", etc that is correct for AR1 too
+        if (corr_type != "AR1") { ## General code for "Matern", etc that is correct for AR1 too (good template ?)
           ## Provides precisionFactorList[[rd]] as expected by .reformat_Qmat_info()
           ## solve(sparse_Qmat) gives the correlation matrix
-          Q_CHMfactor <- Cholesky(sparse_Qmat,LDL=FALSE,perm=FALSE) ## called for each corrPars
+          envir$precisionFactorList[[rd]]$Qmat <- sparse_Qmat # should by symmetric by format (typically dsCMatrix)
+          if (is.null(template <- envir$precisionFactorList[[rd]]$template)) { 
+            Q_CHMfactor <- Cholesky(sparse_Qmat,LDL=FALSE,perm=FALSE) 
+            if (identical(.spaMM.data$options$TRY_update,TRUE) 
+                && ! isDiagonal(sparse_Qmat)) { # protection against silly bugs 
+              envir$precisionFactorList[[rd]]$template <- Q_CHMfactor
+            }
+          } else {
+            Q_CHMfactor <- Matrix::update(template, parent=sparse_Qmat) 
+          }
+          envir$precisionFactorList[[rd]]$chol_Q <- as(Q_CHMfactor, "sparseMatrix") # Linv
           ## fitme sparse_precision has an incomplete symSVD=> corr mattrix not computed, 
           ##    and try(<mat_sqrt_fn>(symSVD=symSVD)) fails. Instead use code always valid:
-          Lunique <- solve(Q_CHMfactor,system="Lt") #  L_Q^{-\top}=LMatrix_correlation 
-          ## Lunique is a dtCMatrix; it is for single correlated effect, and still used in HLfit_body; 
-          ## whether it is used or not in MME_method (sXaug_...), a lot of other code still expects it
-          attr(Lunique,"Q_CHMfactor") <- Q_CHMfactor ## FIXME remove ? limited use that saves only optional sparse triangular solves
-          attr(Lunique, "type") <- "from_Q_CHMfactor"
-          processed$AUGI0_ZX$envir$precisionFactorList[[rd]] <- list(chol_Q=as(Q_CHMfactor, "sparseMatrix"), # Linv
-                                                                     Qmat=sparse_Qmat) # should by symmetric by format (typically dsCMatrix)
+          if (
+            (
+              (is.null(TRY_ZAX <- .spaMM.data$options$TRY_ZAX) && processed$augZXy_cond) # default for augZXy_cond
+              || identical(TRY_ZAX,TRUE) # TRY_ZAX may still be NULL if ! augZXy_cond
+            )
+            && ! processed$AUGI0_ZX$vec_normIMRF[rd]) { # solve may be OK when IMRF is normalized (otherwise more code needed in .normalize_IMRF) 
+            Lunique <- Q_CHMfactor # representation of Lunique, not Lunique itself
+            # note that this has an attribute "type" !  
+          } else {
+            Lunique <- solve(Q_CHMfactor,system="Lt") #  L_Q^{-\top}=LMatrix_correlation 
+            ## Lunique is a dtCMatrix; it is for single correlated effect, and still used in HLfit_body; 
+            #  Keeping it as dtCMatrix might be faster for some operations (but not .tcrossprod) (F I X M E test)
+            ## whether it is used or not in MME_method (sXaug_...), a lot of other code still expects it
+            attr(Lunique,"Q_CHMfactor") <- Q_CHMfactor 
+            attr(Lunique, "type") <- "from_Q_CHMfactor"
+          } 
         }
       } else { ## sparse or dense CORRELATION algorithms
         # this is called through HLCor, hence there must be a dense correlation structure
@@ -216,3 +269,54 @@
     } 
   }
 }
+
+.normalize_IMRF <- function(ZAlist, # already ZA in non-IMRF input (or even IMRF input not to be normalized), 
+                                    #   in contrast to .calc_normalized_ZAlist()
+                            vec_normIMRF, 
+                            Zlist=attr(ZAlist,"Zlist"),
+                            strucList) {
+  AMatrices <- attr(ZAlist,"AMatrices")
+  for (rd in  seq_len(length(ZAlist))) { 
+    if (vec_normIMRF[rd]) { 
+      char_rd <- as.character(rd)
+      colnams <- colnames(Zlist[[rd]]) ## unfortunately I cannot yet assume char_rd here (03/2019)
+      if ( ! setequal(rownames(AMatrices[[char_rd]]), colnams)) {
+        stop(paste0("Any 'A' matrix must have row names that match the levels of the random effects\n",
+                    "(i.e. the colnames of the 'Z' design matrix)"))
+      } # ELSE:       
+      AL <- AMatrices[[char_rd]] %*% strucList[[rd]]
+      invnorm <- 1/sqrt(rowSums(AL^2)) # diag(tcrossprod...)
+      normAL <- .Dvec_times_Matrix(invnorm, AMatrices[[char_rd]])
+      ZAlist[[char_rd]] <- Zlist[[rd]] %id*% normAL[colnams,]
+    }
+  }
+  return(ZAlist) ## with unchanged attributes
+}
+
+.calc_normalized_ZAlist <- function(Zlist, # creates ZA from Z and A, even for non-IMRF
+                            AMatrices,
+                            vec_normIMRF, 
+                            strucList) {
+  if (length(Zlist) && length(AMatrices)) {
+    for (char_rd in  names(Zlist)) { # critically uses names here; Zlist and AMatrices are incomplete lists
+      if ( ! is.null(AMatrices[[char_rd]])) {
+        colnams <- colnames(Zlist[[char_rd]])
+        if ( ! setequal(rownames(AMatrices[[char_rd]]), colnams)) {
+          stop(paste0("Any 'A' matrix must have row names that match the levels of the random effects\n", 
+                      "(i.e. the colnames of the 'Z' design matrix)"))
+        } ## ELSE
+        rd <- as.integer(char_rd) # I cannot yet assume strucList[[char_rd]] (nor vec_normIMRF[char_rd])
+        if (vec_normIMRF[rd]) { 
+          AL <- AMatrices[[char_rd]] %*% strucList[[rd]]
+          invnorm <- 1/sqrt(rowSums(AL^2)) # diag(tcrossprod...)
+          normAL <- .Dvec_times_Matrix(invnorm, AMatrices[[char_rd]])
+          Zlist[[char_rd]] <- Zlist[[char_rd]] %id*% normAL[colnams,]
+        } else {
+          Zlist[[char_rd]] <- Zlist[[char_rd]] %*% AMatrices[[char_rd]][colnams, ]
+        }
+      }
+    }
+    attr(Zlist, "AMatrices") <- AMatrices
+  }
+  return(Zlist) ## with other attributes unchanged
+}
\ No newline at end of file
