@@ -28,31 +28,50 @@
           if (is.null(ARphi)) ARphi <- 1 - 1e-6
           upper$corrPars[[char_rd]][["ARphi"]] <- ARphi
         }
-      } else if (corr_type =="MRF") {
-        if ( ! is.null(.get_cP_stuff(canon.init,"kappa",char_rd))) {
-          KAPPAMAX <- moreargs[[char_rd]]$KAPPAMAX
-          hyper <- init.optim$hyper
-          hyper_map <- attr(hyper,"map")
-          trKappa <- hyper[[hyper_map[it]]]$hy_trK
-          if (is.null(trKappa)) {
+      } else if (corr_type =="IMRF") {
+        KAPPAMAX <- moreargs[[char_rd]]$KAPPAMAX
+        hyper <- init.optim$hyper
+        hyper_map <- attr(hyper,"map")
+        trKappa <- hyper[[hyper_map[it]]]$hy_trK 
+        if (is.null(trKappa)) { ## NOT hyper (independent IMRF terms)    or   fixed hyper
+          if ( ! is.null(.get_cP_stuff(canon.init,"kappa",char_rd))) { ## optimized, independent IMRF term
             kappa <- .get_cP_stuff(user.lower,"kappa",char_rd)
-            if (is.null(kappa)) kappa <- 1e-4
+            if (is.null(kappa)) {
+              kappa <- moreargs[[char_rd]]$minKappa
+              if (is.null(kappa)) kappa <- 1e-4
+            } else {
+              minKappa <- moreargs[[char_rd]]$minKappa
+              if ( ! is.null(minKappa) && kappa<minKappa) {
+                warning("User-provided minimum value of kappa is low. Numerical errors may result.",
+                        immediate.=TRUE)
+              }
+            }
             if (optim.scale=="transformed") {
               lower$corrPars[[char_rd]][["trKappa"]] <- .kappaFn(kappa,KAPPAMAX=KAPPAMAX)
             } else lower$corrPars[[char_rd]][["kappa"]] <- kappa
             kappa <- .get_cP_stuff(user.upper,"kappa",char_rd)
             if (is.null(kappa)) kappa <- KAPPAMAX
+            kappa <- min(KAPPAMAX-1e-6, kappa) # optimization should not try to approach infinity
             if (optim.scale=="transformed") {
               upper$corrPars[[char_rd]][["trKappa"]] <- .kappaFn(kappa,KAPPAMAX=KAPPAMAX)
             } else upper$corrPars[[char_rd]][["kappa"]] <- kappa
-          } else {
-            # slightly inelegant as repeated for several it...
-            lower$hyper[[hyper_map[it]]]$hy_trK <- .kappaFn(1e-4,KAPPAMAX=KAPPAMAX)
-            upper$hyper[[hyper_map[it]]]$hy_trK <- .kappaFn(KAPPAMAX,KAPPAMAX=KAPPAMAX) 
-            lower$hyper[[hyper_map[it]]]$hy_trL <- .dispFn(1e-6)
-            upper$hyper[[hyper_map[it]]]$hy_trL <- .dispFn(1e4) 
-          }
+          } ## else fixed : do nothing
+        } else { ## multIMRF hyper, optimized
+          # slightly inelegant as repeated for several it... (same for lambda)
+          lower$hyper[[hyper_map[it]]]$hy_trK <- .kappaFn(1e-4,KAPPAMAX=KAPPAMAX)
+          upper$hyper[[hyper_map[it]]]$hy_trK <- .kappaFn(KAPPAMAX-1e-6,KAPPAMAX=KAPPAMAX) # .kappaFn(KAPPAMAX,.) is Inf => optimize() stops
         }
+        trLambda <- hyper[[hyper_map[it]]]$hy_trL
+        if ( ! is.null(trLambda)) { ## multIMRF hyper, optimized
+          hy_lam <- user.lower$hyper[[hyper_map[it]]]$hy_lam
+          if (is.null(hy_lam)) hy_lam <- 1e-6
+          lower$hyper[[hyper_map[it]]]$hy_trL <- .dispFn(max(.dispInv(trLambda)/1e4,hy_lam))
+          #
+          hy_lam <- user.upper$hyper[[hyper_map[it]]]$hy_lam
+          if (is.null(hy_lam)) hy_lam <- 1e7
+          upper$hyper[[hyper_map[it]]]$hy_trL <- .dispFn(min(.dispInv(trLambda)*1e4,hy_lam))
+        } # else see general code for lambda or all corr_type's 
+        ## (which means that there are canon.init$lambda slots in and only in case of independent optimized IMRFs)
       } else if (corr_type %in% c("Matern","Cauchy")) { 
         lower_cP <- lower$corrPars[[char_rd]]
         if (is.null(lower_cP)) lower_cP <- list()
