@@ -4,9 +4,13 @@ if (Sys.getenv("_LOCAL_TESTS_")=="TRUE") { ## not on CRAN (lost? see 2.15 How do
     require(pkg, character.only=TRUE, quietly=TRUE)
     # options(error = quote({dump.frames(to.file = TRUE)})) # useful for bugs in .do_TRACE()
     if (interactive())  {
+      # options(error=recover)
+      # spaMM.options(TRY_ZAX=TRUE)
+      # abyss <- matrix(runif(2e7),nrow=1000); gc(reset=TRUE) ## partial control of gc trigger...
+      while (dev.cur()>1L) dev.off()
       op <- devAskNewPage(ask=FALSE)
       testfiles <- dir("C:/home/francois/travail/stats/spaMMplus/spaMM/package/tests/testthat/",pattern="*.R",full.names = TRUE)
-      #oldmaxt <- spaMM.options(example_maxtime=100)
+      #oldmaxt <- spaMM.options(example_maxtime=70)
       timings <- t(sapply(testfiles, function(fich){system.time(source(fich))}))
       #spaMM.options(oldmaxt)
       print(colSums(timings))
