@@ -7,6 +7,12 @@ if(requireNamespace("lme4", quietly = TRUE)) {
   try(testthat::expect_equal(predict(res)[1:6,],predict(res,newdata=sleepstudy[1:6,],re.form= ~ (Days|Subject))[,1])) 
   ## tests of predVar with re.form and ranCoefs in test-devel-predVar-ranCoefs.R
   ## a bit slow but detects many problem: (+ effect of refit$lambda)
+  if (FALSE) { ## and to elicit a minqa bug
+    #spaMM.options(allow_augZXy=TRUE) # make sure of this; even if it's the default
+    spaMM.options(optimizer="bobyqa")
+    # and run next fit... the objective function is called by minqa::bobyqa on a par vector outside the bounds
+    # => the logLik is NaN => the test resu>processed$augZXy_env$objective fails bc 'resu' is NaN.
+  }
   (ares <- fitme(Reaction ~ Days + AR1(1|Days) + (Days|Subject), data = sleepstudy))  ## AR-lambda is ~0 hence lik is flat  wrt ARphi
   testthat::expect_equal(logLik(ares),c(p_v=-875.969672803))
   if (spaMM.getOption("example_maxtime")>1.5) { ## approx time v2.4.129 ten times faster than v2.3.33
