@@ -8,18 +8,15 @@ testthat::expect_equal(attr(predict(fit2, intervals="predVar"),"intervals")[1,1]
 simulate(fit2,nsim=3)
 
 ## check simulation and estimation:
-if (spaMM.getOption("example_maxtime")>105) {
+if (spaMM.getOption("example_maxtime")>105) { ## if NOT sparse_precision
   lll <- Loaloa
   lll$ID <- seq(nrow(lll))
-  tnb <- fitme(I(1+floor(log(1+npos)))~1+(1|ID), data=lll,family=Tnegbin(2))
+  lll$resp <- 1+floor(log(1+lll$npos))
+  tnb <- fitme(resp~1+(1|ID), data=lll,family=Tnegbin(2))
   set.seed(123)
   bla <- simulate(tnb,nsim=50)
-  ecd <- apply(bla,2L,function(newy) {
-    cat(".")
-    locdata <- lll
-    locdata$newy <- newy
-    fitme(newy~1+(1|ID),family=Tnegbin(2), data=locdata)$fixef})
+  ecd <- apply(bla,2L,function(newy) { cat("."); update_resp(tnb,newresp = newy)$fixef })
   # estimand is fixef(tnb) = 0.8398057
-  testthat::expect_true(diff(c(mean(ecd),0.8165989))<1e-6) ## test modified in v2.4.0
+  testthat::expect_true(diff(c(mean(ecd),0.8165908))<1e-6) ## test modified in v2.4.0 and again in 2.5.34
   #plot(ecdf(ecd)) ## consistent with the fitted model from which simulations are drawn
 }
